@@ -239,7 +239,12 @@ class Simulation:
                 return self._rule[(a, b)]
         # If the input rule was a function, with possible kwargs
         elif callable(self._rule):
-            return self._rule(a, b, **self._rule_kwargs)
+            # Make a fresh copy in the case of a dataclass in case the function mutates a, b
+            if dataclasses.is_dataclass(a):
+                a, b = dataclasses.replace(a), dataclasses.replace(b)
+            output = self._rule(a, b, **self._rule_kwargs)
+            # If function just mutates a, b but doesn't return, then return new a, b values
+            return (a, b) if output is None else output
         else:
             raise TypeError("rule must be either a dict or a callable.")
 
@@ -371,6 +376,10 @@ class Simulation:
         # accomodate more rapid snapshots.
         interval_length = min(step_length, int(convergence_step * self.simulator.n))
 
+        # Ensure these lengths are at least a positive number of steps
+        step_length = max(step_length, 1)
+        interval_length = max(interval_length, 1)
+
         next_step = self.simulator.t + step_length
 
         for snapshot in self.snapshots:
@@ -397,6 +406,7 @@ class Simulation:
 
         if len(self.snapshots) == 1 and type(self.snapshots[0]) is TimeUpdate:
             self.snapshots.pop()
+            print()
 
     @property
     def reactions(self):
@@ -598,7 +608,7 @@ class StatePlotter(Snapshot):
             array self.config (indexed by states), matrix * config gives an array
             of counts of categories. Used internally for the update function.
     """
-    def __init__(self, state_map=None):
+    def __init__(self, state_map=None, update_time=0.5):
         """Initializes the StatePlotter.
 
         Args:
@@ -606,12 +616,17 @@ class StatePlotter(Snapshot):
         """
         self._matrix = None
         self.state_map = state_map
+        self.update_time = update_time
 
     def _add_state_map(self, state_map):
         """An internal function called to update self.categories and self.matrix."""
-        self.categories = natsorted(list(set([state_map(state) for state in self.simulation.state_list
-                                    if state_map(state) is not None])),
-                                    key=lambda x:x.__repr__())
+        self.categories = []
+        for state in self.simulation.state_list:
+            if state_map(state) is not None and state_map(state) not in self.categories:
+                self.categories.append(state_map(state))
+        # self.categories = natsorted(list(set([state_map(state) for state in self.simulation.state_list
+        #                             if state_map(state) is not None])),
+        #                             key=lambda x:x.__repr__())
         categories_dict = {j: i for i, j in enumerate(self.categories)}
         self._matrix = np.zeros((len(self.simulation.state_list), len(self.categories)), dtype=np.int64)
         for i, state in enumerate(self.simulation.state_list):
@@ -625,7 +640,6 @@ class StatePlotter(Snapshot):
         If self.state_map gets changed, call initialize to update the barplot to
             show the new set self.categories.
         """
-        self.update_time = 0.2
         self.fig, self.ax = plt.subplots()
         if self.state_map is not None:
             self._add_state_map(self.state_map)
@@ -636,7 +650,6 @@ class StatePlotter(Snapshot):
         if max([len(str(c)) for c in self.categories]) > 2:
             self.ax.set_xticklabels(self.ax.get_xticklabels(), rotation=90, ha='center')
         self.ax.set_ylim(0, self.simulation.simulator.n)
-        self.fig.tight_layout()
 
     def update(self, index=None):
         """Update the heights of all bars in the plot."""
@@ -649,12 +662,15 @@ class StatePlotter(Snapshot):
             rect.set_height(heights[i])
 
         self.ax.set_title(f'Time {self.time}')
+        self.fig.tight_layout()
         self.fig.canvas.draw()
 
 
 def time_trials(rule: Rule, ns: List[int], initial_conditions: Union[Callable, List],
                 convergence_condition: Optional[Callable] = None, convergence_check_interval: float = 0.1,
-                num_trials: int = 100, max_wallclock_time: float = 60 * 60 * 24, **kwargs):
+                num_trials: int = 100, max_wallclock_time: float = 60 * 60 * 24,
+
+                **kwargs):
     """Gathers data about the convergence time of a rule.
 
     Args:
